﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.BioBoost.Evaluators;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.BioBoost.Operators.Moves {

  [StorableClass]
  public class MoveEvaluatorAdapter : SingleSuccessorOperator, ISingleObjectiveMoveEvaluator {

    public IConstrainedValueParameter<ISingleObjectiveEvaluator> EvaluatorParameter {
      get { return (IConstrainedValueParameter<ISingleObjectiveEvaluator>) Parameters["Evaluator"]; }
    }

    public ISingleObjectiveEvaluator Evaluator { get { return EvaluatorParameter.Value; } }

    #region construction & cloning

    [StorableConstructor]
    public MoveEvaluatorAdapter(bool isDeserializing) : base(isDeserializing) {}

    public MoveEvaluatorAdapter(MoveEvaluatorAdapter orig, Cloner cloner) : base(orig, cloner) { }

    public MoveEvaluatorAdapter() {
      Parameters.Add(new LookupParameter<DoubleValue>("MoveQuality", "the move quality"));
      var evaluators = new ItemSet<ISingleObjectiveEvaluator>();
      evaluators.Add(new AggregateEvaluator());
      evaluators.Add(new MonolithicEvaluator());
      Parameters.Add(new ConstrainedValueParameter<ISingleObjectiveEvaluator>("Evaluator", "The actual evaluator.", evaluators));
      EvaluatorParameter.Value = EvaluatorParameter.ValidValues.FirstOrDefault(e => e.GetType() == typeof(MonolithicEvaluator));
    }

    public override IDeepCloneable Clone(Cloner cloner) { return new MoveEvaluatorAdapter(this, cloner); }

    #endregion

    public ILookupParameter<DoubleValue> QualityParameter { get { return Evaluator.QualityParameter; } }

    public ILookupParameter<DoubleValue> MoveQualityParameter { get { return (ILookupParameter<DoubleValue>) Parameters["MoveQuality"]; } }

    public override IOperation Apply() {
      var evaluator = (ISingleObjectiveEvaluator)Evaluator.Clone();
      evaluator.QualityParameter.ActualName = MoveQualityParameter.ActualName;
      return new OperationCollection {
        ExecutionContext.CreateChildOperation(evaluator),
        base.Apply(),
      };
    }
  }
}
